//  IoT電子工作キット
//  5-2.屋外からの通信（プッシュ通知）

// ①ライブラリ、Configファイルを読み込み
#include <WiFiClientSecure.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>
#include "config.h"

// ②利用ライブラリの設定を変更（PubSubClientライブラリは128バイトなのを拡張）
#define MQTT_MAX_PACKET_SIZE 1024

// ③Wi-Fi、MQTTクライアントを設定
WiFiClientSecure wifiClient;
PubSubClient mqttClient(host, 8883, wifiClient);

void setup(void) {
  // ④Serial設定
  Serial.begin(115200);
  Serial.println ( );
  // ⑤LED設定
  pinMode ( LED_PIN, OUTPUT );
}

// ⑥無線Wi-Fi設定
void setup_wifi() {
  // ⑦無線Wi-Fi情報設定
  Serial.println ( "Wi-Fi SetUp" );
  WiFi.config( ip, gateway, subnet, DNS );  // DNSがないとMQTT接続不可
  WiFi.begin ( ssid, password );
  // ⑧Wi-Fi接続処理（接続するまで無限ループ）
  while ( WiFi.waitForConnectResult() != WL_CONNECTED ) {
    // ⑨LEDを１秒毎に点滅する
    ledFlag = !ledFlag;
    digitalWrite(LED_PIN, ledFlag);
    // ⑩１秒間Wait
    delay ( 1000 );
    Serial.print ( "." );
  }
  // ⑪Wi-Fi接続できたのでシリアルモニターにIPアドレス表示
  Serial.print ( "Wi-Fi Connected! IP address: " );
  Serial.println ( WiFi.localIP() );
  // ⑫LED点灯（Wi-Fi接続状態）
  digitalWrite ( LED_PIN, true );
  // ⑬クライアントにCA証明書を設定
  wifiClient.setCACert(beebottle_ca_cert);
}


void loop(void){
  // ⑭Wi-Fi状態を確認し接続状態でなかったらWi-Fi接続処理
  if (WiFi.status() != WL_CONNECTED) {
    setup_wifi();
  }
  // ⑮MQTT状態を確認し接続状態でなかったらMQTT接続処理
  if (!mqttClient.connected()) {
    reconnect();
  }
  // ⑯MQTTクライアント処理
  mqttClient.loop();
}

// ⑰MQTT接続処理
void reconnect() {
  // ⑱MQTT接続状態までループ処理
  while (!mqttClient.connected()) {
    Serial.println("Attempting MQTT connection...");
    // ⑲MQTT設定情報定義
    String username = "token:";
    username += channelToken;
    // ⑳MQTT接続処理
    mqttClient.connect(clientID, username.c_str(), NULL);
    delay(2000);
  }
  Serial.println("MQTT connected");
  // ㉑MQTTメッセージ受信時の処理設定
  mqttClient.setCallback(callback);
  // ㉒MQTTメッセージを受信するTOPICの設定
  mqttClient.subscribe(topic);
}

// ㉓MQTTメッセージ受信時の処理
void callback(char* topic, byte* payload, unsigned int length) {
  // ㉔MQTT受信メッセージを変数に保存
  char recvData[MQTT_MAX_PACKET_SIZE];
  snprintf(recvData, sizeof(recvData), "%s", payload);
  // ㉕MQTT受信メッセージをシリアルモニタ表示
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.println("] ");
  Serial.println(recvData);

  // ㉖受信データのJSON形式を解析し変数へ保存
  StaticJsonBuffer<MQTT_MAX_PACKET_SIZE> jsonBuffer;
  JsonObject& jsonBuf = jsonBuffer.parseObject(recvData);
  // ㉗JSON形式解析が成功しなかったらエラー表示し終了
  if (!jsonBuf.success()) {
    Serial.println("parseObject() failed");
    return;
  }

  // ㉘受信データ（data）を取得保存
  const char* parsedPayload = jsonBuf["data"];
  // ㉙受信データ（data）が存在するか判定
  if (parsedPayload != NULL) {
    Serial.print("payload: ");
    Serial.println(parsedPayload);
    // ㉚受信データ（data）が"led_on"の場合にLEDをON
    if (strcmp(parsedPayload, "led_on") == 0) {
      digitalWrite(LED_PIN, HIGH);
    // ㉛受信データ（data）が"led_off"の場合にLEDをOFF
    } else if (strcmp(parsedPayload, "led_off") == 0) {
      digitalWrite(LED_PIN, LOW);
    }
  }
}
