//  IoT電子工作キット sLab-Remo2
//  6-1.環境センサ付スマートリモコン

// ①ライブラリ、Configファイルを読み込み
#include <WiFiClientSecure.h>
#include <EEPROM.h>
#include <DHT.h>
#include <SPIFFS.h>
#include <ESPAsyncWebServer.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>      // Versionは「5.13.5」
#include "config.h"

// ②利用ライブラリの設定を変更（PubSubClientライブラリは128バイトなのを拡張）
#define MQTT_MAX_PACKET_SIZE 1024
// ③MQTT接続及び人感センサーON/OFF（OFF時はfalse）
bool MQTT_CONNECT = false;
bool MOTION_CHECK = false;
// ④Wi-Fi、MQTTクライアントを設定
WiFiClientSecure wifiClient;            // MQTT用
WiFiClientSecure wifiClientHttps;       // HTTPS用
AsyncWebServer webServer ( 80 );
PubSubClient mqttClient(host, 8883, wifiClient);

// ⑤起動時、最初に実行される処理
void setup(void) {
  // ⑥Serial設定
  Serial.begin(115200);
  Serial.println ( );
  // ⑦SPIFFS開始
  SPIFFS.begin();
  // ⑧EEPROM開始（サイズ指定：ボタン名10個分[65*10]）
  EEPROM.begin(650);
  // ⑨ピンモード設定
  pinMode ( LED_PIN,  OUTPUT );
  pinMode ( IR_R_PIN, INPUT );
  pinMode ( IR_S_PIN, OUTPUT );
  pinMode ( LIGH_SNR, INPUT );
  pinMode ( MOTN_SNR, INPUT );
  // ⑩Wi-Fi接続処理
  setup_wifi();
  // ⑪WebServer設定処理
  setup_webserver();
  Serial.println("##### setup completed #####");
}

// ⑫無線Wi-Fi設定
void setup_wifi() {
  // ⑬無線Wi-Fi情報設定
  Serial.println ( "Wi-Fi SetUp" );
  WiFi.config( ip, gateway, subnet, DNS );  // DNSがないとMQTT接続不可
  WiFi.begin ( ssid, password );
  // ⑭Wi-Fi接続処理（接続するまで無限ループ）
  while ( WiFi.status() != WL_CONNECTED ) {
    // ⑮LEDを１秒毎に点滅する
    ledFlag = !ledFlag;
    digitalWrite(LED_PIN, ledFlag);
    // ⑯１秒間Wait
    delay ( 1000 );
    Serial.print ( "." );
  }
  // ⑰Wi-Fi接続できたのでシリアルモニターにIPアドレス表示
  Serial.print ( "Wi-Fi Connected! IP address: " );
  Serial.println ( WiFi.localIP() );
  // ⑱LED点灯（Wi-Fi接続状態）
  digitalWrite ( LED_PIN, true );
  // ⑲------- MQTT_CONNECTがONの場合のみ クライアントにCA証明書を設定-------
  if ( MQTT_CONNECT ) {
    wifiClient.setCACert(beebottle_ca_cert);
  }
}

// ⑳setup完了後、電源OFFまで繰り返し処理
void loop(void){
  // ㉑現在時刻の取得
  nowtime = millis();

  // ㉒------- MQTT_CONNECTがONの場合のみ -------
  if ( MQTT_CONNECT ) {
    // ㉓MQTT状態を確認し接続状態でなかったらMQTT接続処理
    if ( !mqttClient.connected() ) {
      reconnect();
    }
    // ㉔MQTTクライアント処理
    mqttClient.loop();
  }
  // ㉕------- MOTION_CHECKがONの場合のみ -------
  if ( MOTION_CHECK ) {
    // ㉖1Sec毎にCheck処理
    if ( (long)(nowtime / 1000) != mCheck ) {
      mCheck = (long)(nowtime / 1000);
      // ㉗感知チェック、かつ、モーションセンサー感知状態
      if (motionCheck && digitalRead(MOTN_SNR)) {
        Serial.println( "Detected!" );
        motionCheck = false;        // ㉘感知フラグOFF
        contRemocon ("1");          // ㉙赤外線送信する関数を実行
        iftttMail ();               // ㉚IFTTTでメール送信
      } else {
        Serial.println( "No detected." );
        motionCheck = true;         // ㉛感知チェックON
      }
    }
  }
}

// ㉜MQTT接続処理
void reconnect() {
  // ㉝MQTT接続状態までループ処理
  while (!mqttClient.connected()) {
    Serial.println("Attempting MQTT connection...");
    // ㉞MQTT設定情報定義
    String username = "token:";
    username += channelToken;
    // ㉟MQTT接続処理
    mqttClient.connect(clientID, username.c_str(), NULL);
    delay(2000);
  }
  Serial.println("MQTT connected");
  // ㊱MQTTメッセージ受信時の処理設定
  mqttClient.setCallback(callback);
  // ㊲MQTTメッセージを受信するTOPICの設定
  mqttClient.subscribe(topic);
}

// ㊳MQTTメッセージ受信時の処理
void callback(char* topic, byte* payload, unsigned int length) {
  // ㊴MQTT受信メッセージを変数に保存
  char recvData[MQTT_MAX_PACKET_SIZE];
  snprintf(recvData, sizeof(recvData), "%s", payload);
  // ㊵MQTT受信メッセージをシリアルモニタ表示
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.println("] ");
  Serial.println(recvData);
  // ㊶受信データのJSON形式を解析し変数へ保存
  StaticJsonBuffer<MQTT_MAX_PACKET_SIZE> jsonBuffer;
  JsonObject& jsonBuf = jsonBuffer.parseObject(recvData);
  // ㊷JSON形式解析が成功しなかったらエラー表示し終了
  if (!jsonBuf.success()) {
    Serial.println("parseObject() failed");
    return;
  }
  // ㊸受信データ（data）を取得保存
  const char* parsedPayload = jsonBuf["data"];
  // ㊹受信データ（data）が存在するか判定
  if (parsedPayload != NULL) {
    Serial.print("payload: ");
    Serial.println(parsedPayload);
    // ㊺受信データ（data）に送信リモコン番号
    if (contRemocon( parsedPayload )) {
      Serial.println("MQTT send OK");
    } else {
      Serial.println("MQTT send NG");
    }
  }
}
